Package: BiocCheck
Title: Bioconductor-specific package checks
Version: 1.48.1
Date: 2026-06-30
Authors@R: c(
    person("Bioconductor", "Package Maintainer", ,
        "maintainer@bioconductor.org", "aut"),
    person("Lori", "Shepherd", role = "aut"),
    person("Daniel", "von Twisk", role = "ctb"),
    person("Kevin", "Rue", role = "ctb"),
    person("Marcel", "Ramos", , "marcel.ramos@sph.cuny.edu",
        c("aut", "cre"), c(ORCID = "0000-0002-3242-0582")),
    person("Leonardo", "Collado-Torres", role = "ctb"),
    person("Federico", "Marini", role = "ctb")
  )
Description: BiocCheck guides maintainers through Bioconductor best
    practicies. It runs Bioconductor-specific package checks by searching
    through package code, examples, and vignettes. Maintainers are
    required to address all errors, warnings, and most notes produced.
License: Artistic-2.0
URL: https://github.com/Bioconductor/BiocCheck
BugReports: https://github.com/Bioconductor/BiocCheck/issues
Depends: R (>= 4.4.0)
Imports: BiocBaseUtils, BiocFileCache, BiocManager, biocViews, callr,
        cli, codetools, commonmark, graph, httr2, knitr, methods,
        rvest, stringdist, tools, utils, xml2
Suggests: BiocStyle, devtools, gert, jsonlite, rmarkdown, tinytest,
        usethis
VignetteBuilder: knitr
biocViews: Infrastructure
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
git_url: https://git.bioconductor.org/packages/BiocCheck
git_branch: RELEASE_3_23
git_last_commit: 6029e2b
git_last_commit_date: 2026-06-30
Repository: Bioconductor 3.23
Date/Publication: 2026-07-02
NeedsCompilation: no
Packaged: 2026-07-02 20:37:42 UTC; biocbuild
Author: Bioconductor Package Maintainer [aut],
  Lori Shepherd [aut],
  Daniel von Twisk [ctb],
  Kevin Rue [ctb],
  Marcel Ramos [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3242-0582>),
  Leonardo Collado-Torres [ctb],
  Federico Marini [ctb]
Maintainer: Marcel Ramos <marcel.ramos@sph.cuny.edu>
