Package: testpkg0
Title: bla
Description: blsa
Version: 0.1
Author: Joe Blow
Maintainer: Joe Blow <joe@blow.com>
Depends: R (>= 3.1.0), jsonlite
License: GPL-2
LazyData: true
Imports: devtools, tools
Remotes:
VignetteBuilder: knitr
