%\VignetteIndexEntry{Vignette Title}

```{r echo=FALSE}
session_info()
install_bioc <- function() {
    message("Installing...")
}
```

```{r eval=FALSE}
if (!requireNamespace("BiocCheck", quietly = TRUE))
    BiocManager::install("BiocCheck")
```

```{r eval=TRUE}
## This should trigger checkVigInstalls (2x)
if (!requireNamespace("BiocCheck", quietly = TRUE))
    BiocManager::install("BiocCheck")
## trigger with grep
install_bioc("BiocCheck")
```

```{r}
## This should trigger checkVigBiocInst
if (!requireNamespace("BiocCheck", quietly = TRUE))
    BiocInstaller::biocLite("BiocCheck")
```

```{r eval=FALSE}
## This should not trigger checkVigInstalls
if (!requireNamespace("BiocCheck", quietly = TRUE))
    BiocInstaller::biocLite("BiocCheck")
check_install("BiocCheck")
```

```{r eval=FALSE}
```

```{r eval=FALSE}
```

```{r eval=FALSE}
```

```{r, eval=TRUE}
if (T) {
    cat("bad practice")
}
```

```{r eval=FALSE}
```

```{r eval=FALSE}
```

```{r, child = "testpkg0_child.Rmd"}
```
