\name{internal_too}
\alias{internal_too}
\title{
internal function too
}
\description{
desc
}
\usage{
internal_too()
}
%- maybe also 'usage' for other objects documented here.
\details{
%%  ~~ If necessary, more details than the description above ~~
}
\examples{
    x <- 1:10
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ internal }
