% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BiocCheck-class.R
\name{Message-methods}
\alias{Message-methods}
\alias{setMessage,Message-method}
\alias{setCondition,Message-method}
\alias{getCondition,Message-method}
\title{A list of methods for the Message reference class}
\arguments{
\item{condition}{character(1) One of the three conditions handled: \code{error},
\code{warning}, or \code{note}}

\item{verbose}{logical(1) Whether to output the full text in the check or
only the check name itself in the report}

\item{\dots}{\code{list()} A nested list with the check name as the top level
layer. Second level lists include any \code{help_text} and \code{messages} that are
part of the check.}
}
\value{
An internal \code{R5} Reference Class to handle messages and their
conditions, e.g., for errors, warnings, or notes.
}
\description{
A list of methods for the Message reference class
}
